# -*- coding: utf-8 -*-
import re
import shelve

try:
    from pathlib import Path
    Path().expanduser()
except (ImportError, AttributeError):
    from pathlib2 import Path  # Patch for Python 2.x

import requests
import mechanicalsoup
import demjson
from bs4 import BeautifulSoup

DATA_DIR = Path.home().joinpath(".uricli")
if not DATA_DIR.exists():
    DATA_DIR.mkdir()

SHELVE_PATH = str(DATA_DIR.joinpath('config'))


def restore_session():
    session = requests.session()
    with shelve.open(SHELVE_PATH) as db:
        if not db.get("cookies", None):
            cookies = requests.utils.cookiejar_from_dict(db["cookies"])
            session.cookies = cookies

    return session


def save_session(session):
    cookies = requests.utils.dict_from_cookiejar(session.cookies)
    with shelve.open(SHELVE_PATH) as db:
        db["cookies"] = cookies
        db.sync()


def get_browser():
    browser = mechanicalsoup.StatefulBrowser(session=restore_session())
    return browser


def submit(solution_path, problem_id):
    if problem_id is None:
        problem_id = int(Path(solution_path).resolve().stem)

    with open(solution_path, 'r') as f:
        source_code = f.read()

    language = get_language()

    browser = get_browser()
    browser.open("https://www.urionlinejudge.com.br/judge/en/runs/add")
    ensure_logged_in(browser)

    browser.select_form('form[method="post"]')

    browser['problem_id'] = problem_id
    browser['source_code'] = source_code
    browser['language_id'] = language

    browser.submit_selected()
    print_toast(browser)


def print_toast(browser):
    html = str(browser.get_current_page())
    print("Você pode acompanhar o status da sua submissao no link:\n\033[1;32;48m" + str(
        browser.get_url()) + '\033[1;37;0m')
    try:
        last_status = ""
        while True:
            browser.refresh()
            html = str(browser.get_current_page())
            soup = BeautifulSoup(html, "lxml")
            msg_s = []
            for i in [0, 1, 2, 3, 4, 5, 6, 7, 8]:
                msg_s = soup.findAll('span', attrs={
                    'class': 'answer a-'+str(i)})
                if msg_s:
                    break
            if len(msg_s):
                if msg_s[0].text.strip() == last_status:
                    continue
                print("[status atual] "+msg_s[0].text.replace("-","").strip())
                last_status = msg_s[0].text.strip()
                if "queue" not in last_status:
                    break
            else:
                break
    except Exception as e:
        pass


def set_login(email, password):
    with shelve.open(SHELVE_PATH) as db:
        db['email'] = email
        db['password'] = password
        db['cookies'] = None
        db.sync()


def get_login():
    with shelve.open(SHELVE_PATH) as db:
        if "email" not in db or "password" not in db:
            raise Exception("Must set email and password first.")

        return db['email'], db['password']


def ensure_logged_in(browser):
    email, password = get_login()

    if "/login" in browser.get_url():
        browser.select_form(
            'form[action$="/judge/en/login?redirect=%2Fen%2Fruns%2Fadd"]')
        browser['email'] = email
        browser['password'] = password
        browser['remember_me'] = 1
        browser.submit_selected()

    if "/login" in browser.get_url():
        print_toast(browser)
        raise RuntimeError("Failed to login.")

    save_session(browser.session)


def list_languages():
    browser = get_browser()
    browser.open("https://www.urionlinejudge.com.br/judge/en/runs/add")
    ensure_logged_in(browser)

    browser.select_form('form[method="post"]')
    select = browser.get_current_page().find("select", {"name": "language_id"})
    options = select.find_all("option")
    options.sort(key=lambda a: int(a.attrs['value']))

    return [
        (option.attrs['value'], option.text)
        for option in options
    ]


def get_language():
    with shelve.open(SHELVE_PATH) as db:
        if 'language_id' not in db:
            raise Exception("Must set language first.")
        return db['language_id']


def set_language(language_id):
    with shelve.open(SHELVE_PATH) as db:
        db['language_id'] = language_id



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""URI Online Judge Command Line Interface

Usage:
  uri login
  uri set_language
  uri submit <solution_path> [<problem_id>]
  uri live

Arguments
    <solution_path>     Path to the file containing to solution.
    <problem_id>        URI Online Judge's problem ID

"""

from docopt import docopt
from PyInquirer import prompt


from . import lib
from .live import LiveSession

import json

__version__ = "1.0.1"


def cmd_login():
    questions = [
        {
            'type': 'input',
            'name': 'email',
            'message': 'Email',
        }, {
            'type': 'password',
            'name': 'password',
            'message': 'Password',
        }
    ]
    answers = prompt(questions)

    lib.set_login(answers["email"], answers["password"])


def cmd_set_language():
    language_map = {text: id for (id, text) in lib.list_languages()}

    questions = [
        {
            'type': 'list',
            'name': 'language_key',
            'message': 'Language',
            'choices': language_map.keys()
        }
    ]
    answers = prompt(questions)
    language_key = answers['language_key']
    language_details = json.loads(language_key)
    language_id = language_details['id']

    lib.set_language(language_id)


def cmd_submit(arguments):
    try:
        lib.get_language()
    except BaseException:
        cmd_set_language()  # Ensure language is set first

    solution_path = arguments["<solution_path>"]
    problem_id = arguments["<problem_id>"]

    lib.submit(solution_path, problem_id)


def cmd_live():
    session = LiveSession()
    session.run()


def cmd_help():
    print(__doc__)


def main():
    try:
        arguments = docopt(__doc__, help=False, version=__version__)
    except SystemExit:
        cmd_help()
        exit(1)
        return

    try:
        if arguments["login"]:
            cmd_login()
        elif arguments["set_language"]:
            cmd_set_language()
        elif arguments["submit"]:
            cmd_submit(arguments)
        elif arguments["live"]:
            cmd_live()
        else:
            cmd_help()
    except Exception as e:
        print(e)
        exit(1)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""setup.py: setuptools control."""

import re
from setuptools import setup

version = re.search(
    r'^__version__\s*=\s*"(.*)"',
    open('urigui/uricli.py').read(),
    re.M
).group(1)

with open("README.md", "rb") as f:
    long_descr = f.read().decode("utf-8")

setup(
    name="urigui",
    packages=["urigui"],
    entry_points={
        "console_scripts": ['uri = urigui.uricli:main']
    },
    version=version,
    description="Utility CLI for submitting solutions to URI Online Judge.",
    long_description=long_descr,
    long_description_content_type='text/markdown',
    author="Marcus Vinicius",
    author_email="viniips@hotmail.com",
    install_requires=[
        "docopt==0.6.2",
        "PyInquirer==1.0.2",
        "requests==2.20.0",
        "mechanicalSoup==0.11.0",
        "demjson==2.2.4",
        "websocket-client==0.53.0"
    ],
    url="https://github.com/vininjr/urigui",
    license='Apache 2.0'
)



# -*- coding: utf-8 -*-

import json
import websocket


class LiveSession(object):
    def __init__(self):

        def report_msg(tokens):
            data = {
                "submit_id": tokens[0],
                "problem_id": tokens[1],
                "user_id": tokens[2],
                "user_name": tokens[3],
                "result_code": tokens[4],
                "result_text": tokens[5],
                "language": tokens[6],
                "duration": tokens[7],
                "timestamp": tokens[8],
                "rank": tokens[9],
                "percentage": tokens[10]
            }
            print("User: \"{user_name}\", "
                  "Problem: {problem_id}, "
                  "Language: \"{language}\", "
                  "Result: \"{result_text}\"".format(**data))

        def on_message(ws, message):
            begin = message.index("[")
            if begin >= 0:
                message = message[begin:]
                type_, payload = json.loads(message)
                if type_ == "node":
                    tokens = payload["message"].split(",")
                    report_msg(tokens)
                else:
                    print(payload["message"])

        def on_error(ws, error):
            print(error)

        self.ws = websocket.WebSocketApp("wss://live.urionlinejudge.com.br:8080/socket.io/?EIO=3&transport=websocket",
                                         on_message=on_message,
                                         on_error=on_error)

    def run(self):
        self.ws.run_forever()
